//----------------------------------------------------------------------------
// Copyright 2006 Joe Lowe
//
// Permission is granted to any person obtaining a copy of this Software,
// to deal in the Software without restriction, including the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and sell copies of
// the Software.
//
// The above copyright and permission notice must be left intact in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS WITHOUT WARRANTY.
//----------------------------------------------------------------------------
// file name:  stdint.h
// created:    2006.11.12
//----------------------------------------------------------------------------
#ifndef STDINT_H
#define STDINT_H

// Subset of definitions from ISO C99 stdint.h, for systems/compilers that
// don't supply the header.

#if !defined(SIZE_MAX)
#if defined(_MSC_VER) && !defined(_M_IX86)
// Microsoft had to be different, only modern system/compiler where size of
// long isn't 64 bit on 64 bit addressed systems.
#define SIZE_MAX _UI64_MAX
#elif LONG_MAX == INT_MAX
#define SIZE_MAX UINT_MAX
#else
#define SIZE_MAX ULONG_MAX
#endif
#endif

typedef        signed char  int8_t   ;
typedef      unsigned char  uint8_t  ;
typedef              short  int16_t  ;
typedef     unsigned short  uint16_t ;
typedef                int  int32_t  ;
typedef           unsigned  uint32_t ;
#if LONG_MAX > INT_MAX
typedef               long  int64_t  ;
typedef      unsigned long  uint64_t ;
#elif defined(_MSC_VER)
typedef            __int64  int64_t  ;
typedef   unsigned __int64  uint64_t ;
#else
typedef          long long  int64_t  ;
typedef unsigned long long  uint64_t ;
#endif
#if !defined(_INTPTR_T_DEFINED) && !defined(__intptr_t_defined)
#define _INTPTR_T_DEFINED
#define _UINTPTR_T_DEFINED
#define __intptr_t_defined
#define __uintptr_t_defined
#if SIZE_MAX == UINT_MAX
typedef                int  intptr_t ;
typedef           unsigned  uintptr_t;
#elif LONG_MAX > INT_MAX
typedef               long  intptr_t ;
typedef      unsigned long  uintptr_t;
#elif defined(_MSC_VER)
typedef            __int64  intptr_t ;
typedef   unsigned __int64  uintptr_t;
#else
typedef          long long  intptr_t ;
typedef unsigned long long  uintptr_t;
#endif
#endif

#define    INT8_MIN      SCHAR_MIN
#define    INT8_MAX      SCHAR_MAX
#define   UINT8_MAX      UCHAR_MAX
#define   INT16_MIN       SHRT_MIN
#define   INT16_MAX       SHRT_MAX
#define  UINT16_MAX      USHRT_MAX
#define   INT32_MIN        INT_MIN
#define   INT32_MAX        INT_MAX
#define  UINT32_MAX       UINT_MAX
#if LONG_MAX > INT_MAX
#define   INT64_MIN       LONG_MIN
#define   INT64_MAX       LONG_MAX
#define  UINT64_MAX      ULONG_MAX
#elif defined(_MSC_VER)
#define   INT64_MIN       _I64_MIN
#define   INT64_MAX       _I64_MAX
#define  UINT64_MAX      _UI64_MAX
#else
#define   INT64_MIN      (-9223372036854775807LL-1)
#define   INT64_MAX        9223372036854775807LL
#define  UINT64_MAX        0xFFFFFFFFFFFFFFFFULL
#endif
#if SIZE_MAX == UINT_MAX
#define   INTPTR_MIN       INT_MIN
#define   INTPTR_MAX       INT_MAX
#define  UINTPTR_MAX      UINT_MAX
#elif LONG_MAX > INT_MAX
#define   INTPTR_MIN      LONG_MIN
#define   INTPTR_MAX      LONG_MAX
#define  UINTPTR_MAX     ULONG_MAX
#elif defined(_MSC_VER)
#define   INTPTR_MIN      _I64_MIN
#define   INTPTR_MAX      _I64_MAX
#define  UINTPTR_MAX     _UI64_MAX
#else
#define   INTPTR_MIN     (-9223372036854775807LL-1)
#define   INTPTR_MAX       9223372036854775807LL
#define  UINTPTR_MAX       0xFFFFFFFFFFFFFFFFULL
#endif
#if !defined(PTRDIFF_MIN)
#define PTRDIFF_MIN     INTPTR_MIN
#define PTRDIFF_MAX     INTPTR_MAX
#endif

#define INT8_C(c)   c
#define UINT8_C(c)  c##U
#define INT16_C(c)  c
#define UINT16_C(c) c##U
#define INT32_C(c)  c
#define UINT32_C(c) c##U
#if LONG_MAX > INT_MAX
#define INT64_C(c)  c##L
#define UINT64_C(c) c##UL
#elif defined(_MSC_VER)
#define INT64_C(c)  c##I64
#define UINT64_C(c) c##UI64
#else
#define INT64_C(c)  c##LL
#define UINT64_C(c) c##ULL
#endif

#endif
